#ifndef __HMECMATC_H__
#define __HMECMATC_H__
 
	#include "MTH.h"
	#include "acp_base.h"
 
	#include "cpa_expt.h" 
 
#ifdef __cplusplus
extern "C" {
#endif
 
    /************************************************************************/
    /* Define the handle linked to the structure tdstMecMatCharacteristics  */
    /************************************************************************/
 
    typedef void* DNM_tdxHandleToMecMatCharacteristics;
 
    /* Methods */
 
    /* Invalidate the handle */
    extern CPA_EXPORT void DNM_fn_vInvalidateMatCharacteristics(DNM_tdxHandleToMecMatCharacteristics *_p_hMatCharacteristics);
 
    /* Handle is valid ? */
    extern CPA_EXPORT ACP_tdxBool DNM_fn_bIsMatCharacteristicsValid(DNM_tdxHandleToMecMatCharacteristics _hMatCharacteristics);
 
    /* Create a handle */
    extern CPA_EXPORT DNM_tdxHandleToMecMatCharacteristics DNM_fn_xMatCharacteristicsCreate(void);
 
    /* Destroy a handle */
	extern CPA_EXPORT void DNM_fn_vMatCharacteristicsDestroy(DNM_tdxHandleToMecMatCharacteristics _hOld);
	
    /* Copy a handle */
	extern CPA_EXPORT void DNM_fn_vMatCharacteristicsCopy(DNM_tdxHandleToMecMatCharacteristics _hDst,DNM_tdxHandleToMecMatCharacteristics _hSrc);
 
    extern CPA_EXPORT MTH_tdxReal DNM_fn_xMatCharacteristicsGetSlide(DNM_tdxHandleToMecMatCharacteristics _hMatCharacteristics);
    extern CPA_EXPORT MTH_tdxReal DNM_fn_xMatCharacteristicsSetSlide(DNM_tdxHandleToMecMatCharacteristics _hMatCharacteristics,
                                                MTH_tdxReal _xValue);
 
    extern CPA_EXPORT MTH_tdxReal DNM_fn_xMatCharacteristicsGetRebound(DNM_tdxHandleToMecMatCharacteristics _hMatCharacteristics);
    extern CPA_EXPORT MTH_tdxReal DNM_fn_xMatCharacteristicsSetRebound(DNM_tdxHandleToMecMatCharacteristics _hMatCharacteristics,
                                                MTH_tdxReal _xValue);
 
#ifdef __cplusplus
}
#endif
 
#endif
